#include "../stdafx.h"
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "wincompat.h"

#ifdef _MSC_EXTENSIONS
#ifndef EAGAIN
#define EAGAIN          11
#endif
#define ENOMEM          12
#define ENOSYS          40
#endif

struct pthread {
	void *pt_handle;
	unsigned long pt_id;
};

struct pthread_cond {
	HANDLE ptc_handle;
};

struct pthread_mutex {
	CRITICAL_SECTION ptm_cs;
};

pthread_t pthread_self(void)
{
	return (pthread_t)GetCurrentThreadId();
}

int
pthread_create(pthread_t *pt, const pthread_attr_t *attr,
    void *(*start)(void *), void *arg)
{
	if (!(*pt = malloc(sizeof (**pt))))
		return errno = ENOMEM;
	(*pt)->pt_handle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)start,
	    arg, 0, &(*pt)->pt_id);
	if (!(*pt)->pt_id)
		return errno = EAGAIN;

	return 0;
}

int pthread_up_priority(pthread_t pt)
{
	BOOL resp;
	DWORD err;
	resp = SetThreadPriority(pt->pt_handle, THREAD_PRIORITY_ABOVE_NORMAL);
	if(!resp)
		err = GetLastError();
	return 0;
}

int
pthread_join(pthread_t pt, void **status)
{
	DWORD ec;

	while (GetExitCodeThread(pt->pt_handle, &ec) && ec == STILL_ACTIVE)
		Sleep(100);
	if (status)
		*(DWORD *)status = ec;
	return 0;
}

int
pthread_mutex_init(pthread_mutex_t *ptm, const pthread_mutexattr_t *attr)
{
	if (attr)
		return errno = ENOSYS;
	if (!(*ptm = malloc(sizeof (**ptm))))
		return errno = ENOMEM;
	InitializeCriticalSection(&(*ptm)->ptm_cs);
	return 0;
}

int
pthread_mutex_destroy(pthread_mutex_t *ptm)
{
	DeleteCriticalSection(&(*ptm)->ptm_cs);
	free(*ptm); *ptm = NULL;
	return 0;
}

int
pthread_mutex_lock(pthread_mutex_t *ptm)
{
	EnterCriticalSection(&(*ptm)->ptm_cs);
	return 0;
}

int
pthread_mutex_unlock(pthread_mutex_t *ptm)
{
	LeaveCriticalSection(&(*ptm)->ptm_cs);
	return 0;
}

int
pthread_cond_init(pthread_cond_t *cond, const pthread_condattr_t *attr)
{
	if (attr)
		return errno = ENOSYS;
	if (!(*cond = malloc(sizeof (**cond))))
		return errno = ENOMEM;
		
	(*cond)->ptc_handle = CreateEvent(NULL, FALSE, FALSE, NULL);
	return 0;
}

int
pthread_cond_destroy(pthread_cond_t *cond)
{
	CloseHandle((*cond)->ptc_handle);
	free(*cond); *cond = NULL;
	return 0;
}

int
pthread_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex)
{
	int res;

	pthread_mutex_unlock(mutex);
	res = WaitForSingleObject((*cond)->ptc_handle, INFINITE);
	pthread_mutex_lock(mutex);
	if (res != WAIT_OBJECT_0)
		return errno = ENOSYS;
	return 0;
}

int
pthread_cond_timedwait(pthread_cond_t *cond, pthread_mutex_t *mutex, int *timeout)
{
	int res;

	pthread_mutex_unlock(mutex);
	res = WaitForSingleObject((*cond)->ptc_handle, *timeout);
	pthread_mutex_lock(mutex);
	if (res != WAIT_OBJECT_0)
		return errno = ENOSYS;
	return 0;
}

int
pthread_cond_signal(pthread_cond_t *ptc)
{
	ResetEvent((*ptc)->ptc_handle);
	SetEvent((*ptc)->ptc_handle);
	return 0;
}

int
usleep(unsigned long useconds)
{
	Sleep(useconds / 1000);
	return 0;
}

void
wsa_set_errdesc(char *errdesc, int len)
{
	int err;

	if (!errdesc)
		return;
	switch (err = WSAGetLastError()) {
	case 10038:
		snprintf(errdesc, len, "Socket operation on non-socket");
		return;
	case 10061:
		snprintf(errdesc, len, "Connection refused");
		return;
	default:
		snprintf(errdesc, len, "WSA error %d", err);
		return;
	}
}

/*socketpair for windows, remember to run psocket_init()*/
/*int socketpair(int domain, int type, int protocol, int sv[2]){

    if(protocol != 0){
	            errno = EINVAL;
	return -1;
	}
    if(type != SOCK_STREAM){
	        errno = EINVAL;
	return -1;
	}
    if(domain != AF_UNIX){
	          errno = EINVAL;
	return -1;
	}

    return _pipe(sv, 4096, _O_BINARY );
}*/

int socketpair(int domain, int type, int protocol, int sv[2]){

	// Declare variables
	SOCKET ListenSocket, SendSocket, AcceptSocket;
	struct sockaddr_in saServer;

    if(protocol != 0){
		return -1;
	}
    if(type != SOCK_STREAM){
		return -1;
	}
    if(domain != AF_UNIX){
		return -1;
	}

	//We must use AF_INET, because Windows doesn't support AF_LOCAL sockets

	// Create a listening socket
	ListenSocket = socket(AF_INET, type, protocol);
	if(ListenSocket == INVALID_SOCKET)
		return 1;

	// Set up the sockaddr structure
	saServer.sin_family = AF_INET;
	saServer.sin_addr.s_addr = inet_addr( "127.0.0.1" );
	saServer.sin_port = htons(36987);

	// Bind the listening socket using the
	// information in the sockaddr structure
	bind( ListenSocket,(SOCKADDR*) &saServer, sizeof(saServer) );
	listen( ListenSocket, 1 );

	SendSocket = socket(AF_INET, type, protocol);
	if(SendSocket == INVALID_SOCKET)
		return 1;

	connect( SendSocket, (SOCKADDR*) &saServer, sizeof(saServer) );
	AcceptSocket = accept( ListenSocket, NULL, NULL );
	if(AcceptSocket == INVALID_SOCKET)
		return 1;

	closesocket(ListenSocket);
	sv[0] = SendSocket;
	sv[1] = AcceptSocket;

	return 0;
}


int
strcasecmp(const char *s1, const char *s2)
{
	return lstrcmpi((LPCTSTR)s1, (LPCTSTR)s2);
}

#if ((defined(_WINDOWS) && defined(__GNUC__)) || (defined(WINCE) && _WIN32_WCE < 0x600))
char *
strdup(const char *s)
{
	char *d = NULL;
	int l;

	if (!(d = malloc(l = (strlen(s)+1))))
		return NULL;
	memcpy(d, s, l);
	return d;
}
#endif
#if defined(WINCE)
//very basic, don't support bases other then 10, or endptr, or exponents
__int64 _strtoi64(
   const char *nptr,
   char **endptr,
   int base 
)
{
	__int64 number = 0;
	int ptr = 0;
	int numberpart = 0, neg = 0;

	while(nptr[ptr])
	{
		if(nptr[ptr] == '-')
		{
			if(!numberpart)
				neg = 1;
			else
				break;//fail
		}
		else if(nptr[ptr] == '+')
		{
			if(!numberpart)
				neg = 0;
			else
				break;//fail
		}
		else if(nptr[ptr] >= '0' && nptr[ptr] <= '9')
		{
			numberpart = 1;
			number *= 10;
			number += (nptr[ptr]-'0');
		}
		else
			break;

		ptr++;
	}

	if(neg)
		number = -number;
	
	return number;
}
#endif
